#include <Wire.h>
#include <U8glib.h>

const int lrclkPin = 2;       // LRCLK pin
const int usbPin = 8;         // USB input pin
const int coaxialPin = 9;     // COAXIAL input pin
const int bluetoothPin = 10;  // BLUETOOTH input pin

const unsigned long measurementInterval = 1000000; // Measurement time in microseconds (1 second)
const unsigned long updateInterval = 30000;        // Update interval in milliseconds (30 seconds)

volatile unsigned long risingEdges; // Variable to count rising edges
volatile float sampleRate;         // Variable to store sample rate

U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_NO_ACK); // OLED display object

unsigned long previousMillis = 0; // Stores the last time the display was updated
unsigned long lastMeasurementTime = 0; // Stores the last time a measurement was taken

void normalizeSampleRate() {
  if (sampleRate > 40 && sampleRate <= 46) {
    sampleRate = 44.1;
  }
  else if (sampleRate > 46 && sampleRate <= 50) {
    sampleRate = 48;
  }
  else if (sampleRate > 50 && sampleRate <= 92) {
    sampleRate = 88.2;
  }
  else if (sampleRate > 92 && sampleRate <= 100) {
    sampleRate = 96;
  }
  else if (sampleRate > 110 && sampleRate <= 184.2) {
    sampleRate = 176.4;
  }
  else if (sampleRate > 184 && sampleRate <= 200) {
    sampleRate = 192;
  }
  else if (sampleRate > 200 && sampleRate <= 365) {
    sampleRate = 352.8;
  }
  else if (sampleRate > 365 && sampleRate <= 400) {
    sampleRate = 384;
  }
}

void setup() {
  u8g.begin();   // Initialize OLED display
  Wire.begin();  // Initialize I2C communication
  pinMode(lrclkPin, INPUT);          // Set LRCLK pin as input
  pinMode(usbPin, INPUT_PULLUP);     // Set USB pin as input with internal pull-up resistor
  pinMode(coaxialPin, INPUT_PULLUP); // Set COAXIAL pin as input with internal pull-up resistor
  pinMode(bluetoothPin, INPUT_PULLUP); // Set BLUETOOTH pin as input with internal pull-up resistor
  attachInterrupt(digitalPinToInterrupt(lrclkPin), handleLRCLK, RISING); // Attach ISR for rising edge of LRCLK
}

void loop() {
  unsigned long currentMillis = millis();

  // Check if 30 seconds have passed since the last display update
  if (currentMillis - previousMillis >= updateInterval) {
    previousMillis = currentMillis;
    checkAndUpdateSampleRate();
  }

  // Non-blocking wait for measurement interval
  if (micros() - lastMeasurementTime >= measurementInterval) {
    lastMeasurementTime = micros();
    calculateSampleRate();
  }
}

void checkAndUpdateSampleRate() {
  // Update the OLED display with the current sample rate
  u8g.firstPage();
  do {
    draw(sampleRate);
  } while (u8g.nextPage());
}

void calculateSampleRate() {
  noInterrupts();         // Disable interrupts to read risingEdges safely
  sampleRate = (float)risingEdges / ((float)measurementInterval / 1000000.0 * 2); // Calculate sample rate
  sampleRate /= 1000;     // Convert to kHz
  risingEdges = 0;        // Reset risingEdges count for the next measurement
  normalizeSampleRate();  // Normalize the sample rate
  interrupts();           // Re-enable interrupts
}

void draw(float sampleRate) {
  u8g.setFont(u8g_font_profont12); // Use profont12 font

  // Read the status of pins USB, COAXIAL, BLUETOOTH
  bool usbStatus = digitalRead(usbPin) == LOW;
  bool coaxialStatus = digitalRead(coaxialPin) == LOW;
  bool bluetoothStatus = digitalRead(bluetoothPin) == LOW;

  // Display input source based on pin status
  u8g.setPrintPos(0, 10);
  if (usbStatus) {
    u8g.print("INPUT = USB");
  } 
  if (coaxialStatus) {
    u8g.print("INPUT = COAXIAL");
  } 
  if (bluetoothStatus) {
    u8g.print("INPUT = BLUETOOTH");
  }

  // Display sample rate
  char buffer[6]; // Adjusted buffer size for better safety
  dtostrf(sampleRate, 4, 1, buffer); // Use 1 decimal place
  u8g.drawStr(0, 30, "Sample Rate:");
  u8g.drawStr(0, 50, buffer);
  u8g.drawStr(60, 50, "kHz"); // Display in kHz
}

void handleLRCLK() {
  risingEdges++; // Increment rising edges count
}

/* 
  Code by: M Rehan Abid
  This code is provided as a gift (freeware) to the DIY community and can be used freely. 
  Function of normalize sample rate has been provided by miro1360 from diyaudio.com
  The author requests that the contributors' names should not be removed. 
  For commercial use, please send an email to rehanabid@gmail.com and seek consent.
*/
#include <Wire.h>
#include <U8glib.h>

const int lrclkPin = 2;       // LRCLK pin
const int usbPin = 8;         // USB input pin
const int coaxialPin = 9;     // COAXIAL input pin
const int bluetoothPin = 10;  // BLUETOOTH input pin

const unsigned long measurementInterval = 1000000; // Measurement time in microseconds (1 second)
const unsigned long updateInterval = 30000;        // Update interval in milliseconds (30 seconds)

unsigned long risingEdges = 0; // Variable to count rising edges
float sampleRate = 0.0;        // Variable to store sample rate

U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_NO_ACK); // OLED display object

unsigned long previousMillis = 0; // Stores the last time the display was updated
unsigned long lastMeasurementTime = 0; // Stores the last time a measurement was taken

bool lastLRCLKState = LOW; // Last state of LRCLK pin

void normalizeSampleRate() {
  if (sampleRate > 40 && sampleRate <= 46) {
    sampleRate = 44.1;
  } else if (sampleRate > 46 && sampleRate <= 50) {
    sampleRate = 48;
  } else if (sampleRate > 50 && sampleRate <= 92) {
    sampleRate = 88.2;
  } else if (sampleRate > 92 && sampleRate <= 100) {
    sampleRate = 96;
  } else if (sampleRate > 110 && sampleRate <= 184.2) {
    sampleRate = 176.4;
  } else if (sampleRate > 184 && sampleRate <= 200) {
    sampleRate = 192;
  } else if (sampleRate > 200 && sampleRate <= 365) {
    sampleRate = 352.8;
  } else if (sampleRate > 365 && sampleRate <= 400) {
    sampleRate = 384;
  }
}

void setup() {
  u8g.begin();   // Initialize OLED display
  Wire.begin();  // Initialize I2C communication
  pinMode(lrclkPin, INPUT);          // Set LRCLK pin as input
  pinMode(usbPin, INPUT_PULLUP);     // Set USB pin as input with internal pull-up resistor
  pinMode(coaxialPin, INPUT_PULLUP); // Set COAXIAL pin as input with internal pull-up resistor
  pinMode(bluetoothPin, INPUT_PULLUP); // Set BLUETOOTH pin as input with internal pull-up resistor
}

void loop() {
  unsigned long currentMillis = millis();

  // Poll the LRCLK pin for rising edges
  bool currentLRCLKState = digitalRead(lrclkPin);
  if (currentLRCLKState == HIGH && lastLRCLKState == LOW) {
    risingEdges++;
  }
  lastLRCLKState = currentLRCLKState;

  // Check if 30 seconds have passed since the last display update
  if (currentMillis - previousMillis >= updateInterval) {
    previousMillis = currentMillis;
    checkAndUpdateSampleRate();
  }

  // Non-blocking wait for measurement interval
  if (micros() - lastMeasurementTime >= measurementInterval) {
    lastMeasurementTime = micros();
    calculateSampleRate();
  }
}

void checkAndUpdateSampleRate() {
  // Update the OLED display with the current sample rate
  u8g.firstPage();
  do {
    draw(sampleRate);
  } while (u8g.nextPage());
}

void calculateSampleRate() {
  sampleRate = (float)risingEdges / ((float)measurementInterval / 1000000.0 * 2); // Calculate sample rate
  sampleRate /= 1000;     // Convert to kHz
  risingEdges = 0;        // Reset risingEdges count for the next measurement
  normalizeSampleRate();  // Normalize the sample rate
}

void draw(float sampleRate) {
  u8g.setFont(u8g_font_profont12); // Use profont12 font

  // Read the status of pins USB, COAXIAL, BLUETOOTH
  bool usbStatus = digitalRead(usbPin) == LOW;
  bool coaxialStatus = digitalRead(coaxialPin) == LOW;
  bool bluetoothStatus = digitalRead(bluetoothPin) == LOW;

  // Display input source based on pin status
  u8g.setPrintPos(0, 10);
  if (usbStatus) {
    u8g.print("INPUT = USB");
  } 
  if (coaxialStatus) {
    u8g.print("INPUT = COAXIAL");
  } 
  if (bluetoothStatus) {
    u8g.print("INPUT = BLUETOOTH");
  }

  // Display sample rate
  char buffer[6]; // Adjusted buffer size for better safety
  dtostrf(sampleRate, 4, 1, buffer); // Use 1 decimal place
  u8g.drawStr(0, 30, "Sample Rate:");
  u8g.drawStr(0, 50, buffer);
  u8g.drawStr(60, 50, "kHz"); // Display in kHz
}

/* 
  Code by: M Rehan Abid
  This code is provided as a gift (freeware) to the DIY community and can be used freely. 
  Function of normalize sample rate has been provided by miro1360 from diyaudio.com
  The author requests that the contributors' names should not be removed. 
  For commercial use, please send an email to rehanabid@gmail.com and seek consent.
*/
